/*
Copyright (C) 2011 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
 */
package org.harctoolbox.IrpMaster;

import java.util.ArrayList;

/**
 * This class ...
 *
 * @author Bengt Martensson
 */
public class BitStream extends PrimitiveIrStreamItem {
    
    public final static int maxLength = Long.SIZE;
    private int length;
    private long data;
    
    @Override
    public String toString() {
        return "BitStream, length = " + length + ", data = " + data + " = " + Long.toBinaryString(data);
    }
    
    public BitStream(Protocol env) {
        super(env);
        data = 0;
        length = 0;
    }
    
    public void add(BitField bitField) {
        add(bitField, environment.getBitDirection());
    }
    
    public void add(BitField bitField, BitDirection bitDirection) {
        if (length + bitField.getWidth() > maxLength)
            throw new RuntimeException("Implementation limit in BitStream exceeded.");
        
        long newData = this.environment.getBitDirection() == BitDirection.msb
                ? bitField.toLong()
                : IrpUtils.reverse(bitField.toLong(), bitField.getWidth());
        
        data = data << bitField.getWidth() | newData;
        length += bitField.getWidth();
    }

    /** Extracts bit n*chunksize .. (n+1)*chunksize-1 */
    private int getChunkNo(int n, int chunksize) {
        if (n < 0 || (length > 0 && (n+1)*chunksize-1 >= length))
            throw new IndexOutOfBoundsException("Illegal bit " + n + " in getChunkNo");
        int chunk = (int)(data >> n*chunksize) & ((1 << chunksize)- 1);
        return chunk;
    }
    
    @Override
    public ArrayList<PrimitiveIrStreamItem> evaluate(BitSpec bitSpec) throws UnassignedException, IncompatibleArgumentException {
        debugBegin();
        if (bitSpec == null)
            throw new UnassignedException("BitStream " + toString() + " has no associated BitSpec, cannot compute IRStream");
        ArrayList<PrimitiveIrStreamItem> list = new ArrayList<PrimitiveIrStreamItem>();
        if (length % bitSpec.getChunkSize() != 0)
            throw new IncompatibleArgumentException("chunksize (= " + bitSpec.getChunkSize() + ") does not divide bitstream length (= " + length + ").");
        
        int noChunks = length/bitSpec.getChunkSize();
        for (int n = 0; n < noChunks; n++) {
            int chunkNo = noChunks - n - 1;
            PrimaryIrStream irs = bitSpec.getBitIrsteam(getChunkNo(chunkNo, bitSpec.getChunkSize()));
            ArrayList<PrimitiveIrStreamItem> items = irs.evaluate(null);
            list.addAll(items);
        }
        Debug.debugBitStream(toString());
        debugEnd(list);
        return list;
    }

    @Override
    public boolean isEmpty() {
        return length == 0;
    }
}
